;;; -*- mode: lisp -*-
;;;
;;; lj-util.lisp --- utility functions for use with lj
;;;
;;; Copyright (C) 2001 by Mark Simpson
;;;
;;; Author: Mark Simpson <damned@rover>
;;; This program is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2, or (at your option)
;;; any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, you can either send email to this
;;; program's maintainer or write to: The Free Software Foundation,
;;; Inc.; 675 Massachusetts Avenue; Cambridge, MA 02139, USA.
;;;
;;; Commentary: 

;;; Code:

(eval-when (:compile-toplevel :load-toplevel :execute)
  (require "lj-pkg" (logical-pathname "livejournal:lj-pkg")))

(in-package "LJ")

(defun read-file (file)
  (with-open-file (stream file :direction :input)
                  (apply #'concatenate 'string
                         (loop for line = (read-line stream nil nil)
                               while line do collect (concatenate
                                                      'string
                                                      line
                                                      '(#\Newline))))))

(defun encode-str (str)
  (apply #'concatenate 'string
         (loop for char in (coerce str 'list)
               collect (cond ((alphanumericp char) (list char))
                             ((char= char #\Space) (list #\+))
                             (t (format nil "%~2,'0x" (char-int char)))))))

;; stolen/borrowed from clocc:port
(defstruct hostent
  "see gethostbyname(3) for details"
  (name "" :type simple-string) ; canonical name of host
  (aliases nil :type list)      ; alias list
  (addr-list nil :type list)    ; list of addresses
  (addr-type 2 :type fixnum))   ; host address type

;; stolen/borrowed from clocc:port
(defun resolve-host-ipaddr (host)
  "Call gethostbyname(3) or gethostbyaddr(3)."
  #+allegro
  (let* ((ipaddr
          (etypecase host
            (string
             (if (every (lambda (ch) (or (char= ch #\.) (digit-char-p ch)))
                        host)
                 (socket:dotted-to-ipaddr host)
                 (socket:lookup-hostname host)))
            (integer host)))
         (name (socket:ipaddr-to-hostname ipaddr)))
    (make-hostent :name name :addr-list
                  (list (socket:ipaddr-to-dotted ipaddr))))
  #+(and clisp syscalls)
  (let ((he (posix:resolve-host-ipaddr host)))
    (make-hostent :name (posix::hostent-name he)
                  :aliases (posix::hostent-aliases he)
                  :addr-list (posix::hostent-addr-list he)
                  :addr-type (posix::hostent-addrtype he)))
  #+cmu
  (let ((he (ext:lookup-host-entry host)))
    (make-hostent :name (ext:host-entry-name he)
                  :aliases (ext:host-entry-aliases he)
                  :addr-list (mapcar #'ipaddr-to-dotted
                                     (ext:host-entry-addr-list he))
                  :addr-type (ext::host-entry-addr-type he)))
  #+gcl (make-hostent :name (or (si:hostid-to-hostname host) host)
                      :addr-list (list (si:hostname-to-hostid host)))
  #+lispworks
  (multiple-value-bind (name addr aliases)
      (comm:get-host-entry host :fields '(:name :address :aliases))
    (make-hostent :name name :addr-list (list (ipaddr-to-dotted addr))
                  :aliases aliases))
  #-(or allegro (and clisp syscalls) cmu gcl lispworks)
  (error 'not-implemented :proc (list 'resolve-host-ipaddr host)))

;; stolen/borrowed from clocc:port
(defun open-socket (host port &optional bin)
  "Open a socket connection to HOST at PORT."
  (declare (type (or integer string) host) (fixnum port) (type boolean bin))
  (let ((host (etypecase host
                (string host)
                (integer (hostent-name (resolve-host-ipaddr host))))))
    #+allegro (socket:make-socket :remote-host host :remote-port port
                                  :format (if bin :binary :text))
    #+clisp (#+lisp=cl ext:socket-connect #-lisp=cl lisp:socket-connect
                       port host :element-type
                       (if bin '(unsigned-byte 8) 'character))
    #+cmu (sys:make-fd-stream (ext:connect-to-inet-socket host port)
                              :input t :output t :element-type
                              (if bin '(unsigned-byte 8) 'character))
    #+gcl (si:make-socket-stream host port bin) ; FIXME
    #+lispworks (comm:open-tcp-stream host port :direction :io :element-type
                                      (if bin 'unsigned-byte 'base-char))
    #-(or allegro clisp cmu gcl lispworks)
    (error 'not-implemented :proc (list 'open-socket host port bin))))

(defun close-socket (sock) (close sock))

(defmacro with-open-socket ((var (host port) &key (binary nil)) &body body)
  `(let ((,var (open-socket ,host ,port ,binary)))
     (unwind-protect
         (progn ,@body)
       (close-socket ,var))))

(defun encode-html (str)
  (apply #'concatenate 'string
         (loop for char in (coerce str 'list)
               collect (case char
                         (#\< "&lt")
                         (#\> "&gt")
                         (t (list char))))))

;; useful for getting values out of the hash table... all values in the
;; table are strings.
(defun string->date (str)
  "We know/assume that the date is in the form: yyyy-mm-dd hh:mm[:ss]"
  (with-input-from-string
   (stream (substitute-if #\Space (complement #'digit-char-p) str))
   (let ((year (read stream))
         (mon (read stream))
         (day (read stream))
         (hour (read stream))
         (min (read stream))
         (sec (ignore-errors (read stream))))
     (encode-universal-time (or sec 0) min hour day mon year))))

(defun get-info (tag data &key (type 'string)
                     (default-value (ecase type
                                      (string "")
                                      ((num date) 0))))
  (multiple-value-bind (val foundp) (gethash tag data)
    (if foundp (ecase type
                 (string val)
                 (num (read-from-string val))
                 (date (string->date val)))
      default-value)))

(defun get-nth-info (pre-tag n post-tag data &rest other-keys)
  (apply #'get-info
         (format nil "~A_~D_~A" pre-tag n post-tag)
         data other-keys))

(provide 'lj-util)

;;; lj-util.lisp ends here